function [g,pol] = espira1appr(y,Nexp)
%
%  Input: y - vector of sumples
%         Nexp - wanted order of the constructed exponential sum of good
%         approximation
%         
% Output: g vector of coefficients in the proper exponential sum 
%         pol vectors of poles, zeros of the rational function 


F=fft(y);
M = length(F); % number of modified DFT values
Z=0:M-1; Z=exp(2*pi*1i.*Z/M);  %set knots on the unit circle
Z=Z(:); F=F(:); % work with column vectors
F=F.*Z.^(-1); % modify the DFT values

% We use further AAA algorithm developed in 
%Y. Nakatsukasa, O. Sete, and L.N. Trefethen. The AAA algorithm for rational approximation. SIAM J. Sci. Comput., 40(3):A1494–A1522, 2018.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SF = spdiags(F,0,M,M); % diagonal matrix with values of F
J = 1:M; z = []; f = []; J=J(:); 
C = []; errvec = []; R=zeros(M,1); R=R(:); % initializations

% find the coefficients with largest absolute value
[R1,k1] = max(abs(F-R)); j1=find(J==k1);
R = R+ R1;                % initial rational approximation
z=[z;Z(j1)];              %update sets
f=[f;F(j1)];
J(J==j1)=[];              % update index set
C=[C 1./(Z-Z(j1))];       % first column of Cauchy matrix

err = norm(F(J)-R(J),inf);
errvec = [errvec; err]; % max error at sample points


for m = 2:Nexp+1   % main loop
[~,j] = max(abs(F(J)-R(J)));  % select next support point
z = [z; Z(J(j))];
f = [f; F(J(j))]; % update support points, data values
C = [C 1./(Z-Z(J(j)))];  % next column of Cauchy matrix
J(j) = []; % update index vector
Sf = diag(f); % right scaling matrix
A=SF*C-C*Sf; % Loewner matrix
A=A(J,:);
[~,G,V] = svd(A); % SVD of A
w = V(:,m);  % last singular vector of A
N = C*(w.*f); D = C*w; % numerator and denominator
R = F;
R(J) = N(J)./D(J); % rational approximation
err = norm(F(J)-R(J),inf);
errvec = [errvec; err]; % max error at sample points
end
% end AAA algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
j2=find(abs(w)<=1e-13); %find possible zero components in w
w(j2)=[];              %remove zero components in w
z(j2)=[];              % update corresponding interpolation points
f(j2)=[];              % update corresponding values

[g,AB,pol] = prz(F,Z,z,f,w,M); % poles and zeros and partial decomposition


function [g,AB,pol] = prz(F,Z,z,f,w,M) 
% compute poles pol of rational function (wanted parameters)
% AB complex nominator vector in partial fraction decomposition
% g wanted parameters
m = length(w); 
B = eye(m+1);
B(1,1) = 0;
E = [0 w.'; ones(m,1) diag(z)];
pol = eig(E,B); pol = pol(~isinf(pol)); %  poles
%a=log(pol);

CC=(-1)./bsxfun(@minus,z, pol.');
AB=linsolve(CC,f);
g=-AB./(1-pol.^(M));

disp('condition number of the Cauchy matrix for ESPIRA I');
conditionCC=cond(CC);
disp(conditionCC);
 

