function [g,pol] = espira2apprmod(y,Nexp)
%
%  Input: y - vector of sumples
%         Nexp - wanted order of the constructed exponential sum of good
%         approximation
%         
% Output: g vector of coefficients in the proper exponential sum 
%         pol vectors of poles, zeros of the rational function 

F=fft(y);
M = length(F);                  % number of DFT values
Z=0:M-1; Z=exp(2*pi*1i.*Z/M);  %set knots on the unit circle
Z=Z(:); F1=F(:);               % work with column vectors
F=F.*Z.^(-1);                  % modify the DFT values

% We use further AAA algorithm developed in 
%Y. Nakatsukasa, O. Sete, and L.N. Trefethen. The AAA algorithm for rational approximation. SIAM J. Sci. Comput., 40(3):A1494???A1522, 2018.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SF = spdiags(F,0,M,M);         % diagonal matrix with values F
SF1 = spdiags(F1,0,M,M);       % diagonal matrix with values F1
J = 1:M; z = []; f = []; J=J(:); 
f1=[];                         %new for second matrix
C = []; errvec = []; R=zeros(M,1); R=R(:); % initializations

% find the coefficient with largest absolute value
[R1,k1] = max(abs(F-R)); j1=find(J==k1);
R = R+ R1;                % initial rational approximation
z=[z;Z(j1)];              %update sets
f=[f;F(j1)];
f1=[f1;F1(j1)];
J(J==j1)=[];              % update index set
C=[C 1./(Z-Z(j1))];       % first column of Cauchy matrix
err = norm(F(J)-R(J),inf);
errvec = [errvec; err];   % max error at sample points

for m = 2:Nexp+1              % main loop
[~,j] = max(abs(F(J)-R(J)));  % select next support point
z = [z; Z(J(j))];
f = [f; F(J(j))];             % update support points, data values
f1= [f1; F1(J(j))];           % update for second matrix
C = [C 1./(Z-Z(J(j)))];       % next column of Cauchy matrix
J(j) = [];                    % update index vector
Sf = diag(f);                 % right scaling matrix
A=SF*C-C*Sf;                  % Loewner matrix
A=A(J,:);                     % reduce rows
[~,G,V] = svd(A);             % SVD of A
w = V(:,m);                   % last singular vector of A
N = C*(w.*f); D = C*w;        % numerator and denominator
R = F;
R(J) = N(J)./D(J);            % rational approximation
err = norm(F(J)-R(J),inf);
errvec = [errvec; err];       % max error at sample points
end
% end AAA algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sf1=diag(f1);                 % right scaling matrix, second matrix
B=SF1*C-C*Sf1;                % second Loewner matrix  
B=B(J,:);                     % reduce rows
[~,G,V]=svd(B);
[N1,N2]=size(B);
A1=[A,B];                     % matrix pair for matrix pencil
[~,G,V]=svd(A1);               % joint SVD
%M1=rank(G,10^(-8));           % fix number of terms
M1=Nexp;
V=V(:,1:M1)';                 % reduce rows in V matrix 
V1=V(1:M1,1:N2);              % first matrix for matrix pencil
V2=V(1:M1,N2+1:2*N2);         % second matrix for matrix pencil
pol=eig(pinv(V1.')*V2.');       % solve matrix pencil problem

% Z1=zeros(M,length(pol));     % Compute Vandermonde matrix
% for i=1:length(pol)
%    Z1(:,i)=transpose(pol(i).^(0:M-1)); 
% end
% 
% g=Z1\y;                        % compute coefficients g
% 
% disp('condition number of the Vandermonde matrix for ESPIRA II');
% conditionZ=cond(Z1);
% disp(conditionZ);

CC=(-1)./bsxfun(@minus,z, pol.');
AB=linsolve(CC,f);
g=-AB./(1-pol.^(M));            % compute coefficients g

disp('condition number of the Cauchy matrix for ESPIRA II');
conditionCC=cond(CC);
disp(conditionCC);




 
 
