clc
clear all
close all
format long

%construction of exponential sum of good approximation for the function
%1/(1+x) - Example 6.1 in 
% Derevianko, Plonka, Petz: From ESPRIT to ESPIRA: Estimation of Signal
% Parameters by Iterative Rational Approximation 

 
Nexp=5; % order of the constructed exponential sum
N=60;
L=10;

sf=1/(2*N);
x=0:0.001:1;


% computation of function values
y=[];
for j=1:2*N
    y=[y; 1/(1+sf*(j-1))];  
end


[g,pol]=espira1appr(y,Nexp);   % espira I
[g1,pol1]=mpmappr(y,Nexp,L);     % mpm 
[g2,pol2]=espira2appr(y,Nexp);  % espira II
[g3,pol3]=espritappr(y,Nexp,L);  % esprit


y=1./(1+x);

yes1=statio(g,pol,2*N*x);
ympm=statio(g1,pol1,2*N*x);
yes2=statio(g2,pol2,2*N*x);
yespr=statio(g3,pol3,2*N*x);

disp('errors in maximum norm')
error_espira1=norm(y-yes1,inf)
error_mpm=norm(y-ympm,inf)
error_esprit=norm(y-yespr,inf)
error_espira2=norm(y-yes2,inf)

figure
abb=plot(x, log10(abs(y-yes1)));
set(abb,'linewidth',2); axis([0,1,-15,-8]); set(gca,'fontsize',18); print -depsc bsp22.eps;
hold on
abb=plot(x, log10(abs(y-ympm)));
set(abb,'linewidth',2); axis([0,1,-15,-8]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for: ESPIRA I - blue, MPM - red')
hold off

figure
abb=plot(x, log10(abs(y-yes2)));
set(abb,'linewidth',2); axis([0,1,-15,-8]); set(gca,'fontsize',18); print -depsc bsp22.eps;
hold on
abb=plot(x, log10(abs(y-yespr)));
set(abb,'linewidth',2); axis([0,1,-15,-8]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for: ESPIRA II - blue, ESPRIT - red')
hold off




function f=statio(g1,knot1,x)
[N,~]=size(g1);
f=0;
for k=1:N
f= f+ g1(k)*knot1(k).^x;
end
end





