clc
clear all
close all
format long

% construction of exponential sum of good approximation for the Bessel 
% function of the first kind J_0(100\pi t)
% Example 6.2 of the paper 
% Derevianko, Plonka, Petz: From ESPRIT to ESPIRA: Estimation of  Signal
% Parameters by Iterative Rational Approximation
% We use L=250  for MPM and ESPRIT

Nexp=28; % order of the constructed exponential sum
N=515;   % number of used equidistant function values in [0,1]
L=250;   % uper bound for MPM and ESPRIT

b=100*pi; % scaling factor of the Bessel function 
sf=1/(2*N);
x=0:0.001:1;

% computation of function values
y=[];
for j=1:2*N
     y=[y; besselj(0,sf*(j-1)*b)];
end


[g,pol]=espira1appr(y,Nexp);   % espira I
[g1,pol1]=mpmappr(y,Nexp,L);     % mpm 
[g2,pol2]=espira2appr(y,Nexp);  % espira II
[g3,pol3]=espritappr(y,Nexp,L);  % esprit


a=log(pol);
a1=log(pol1);
a2=log(pol2);
a3=log(pol3);


yb=besselj(0,b*x);

yes1=statio(g,a,2*N*x);
ympm=statio(g1,a1,2*N*x);
yes2=statio(g2,a2,2*N*x);
yespr=statio(g3,a3,2*N*x);

disp('errors in maximum norm')
error_espira1=norm(yb-yes1,inf)
error_mpm=norm(yb-ympm,inf)
error_esprit=norm(yb-yespr,inf)
error_espira2=norm(yb-yes2,inf)


yenv=statio1(g,a,2*N*x); % construction of envelope functions

figure
abb=plot(x, yenv,':','Color',[0,0.4,0.7]);
set(abb,'linewidth',2); axis([0,1,-1.1,1.1]); set(gca,'fontsize',18); print -depsc bsp22.eps;
hold on
abb=plot(x, -yenv,':','Color',[0,0.4,0.7]);
set(abb,'linewidth',2); axis([0,1,-1.1,1.1]); set(gca,'fontsize',18); print -depsc bsp22.eps;
hold on
abb=plot(x, yb, 'Color',[0,0.4,0.7]);
set(abb,'linewidth',2); axis([0,1,-1.1,1.1]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Bessel function with its envelopes')
hold off


figure
abb=plot(x, log10(abs(yb-yes1)));
set(abb,'linewidth',2); axis([0,1,-15,-9]); set(gca,'fontsize',18); print -depsc bsp22.eps;
hold on
abb=plot(x, log10(abs(yb-ympm)));
set(abb,'linewidth',2); axis([0,1,-15,-9]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for: ESPIRA I - blue, MPM - red')
hold off

figure
abb=plot(x, log10(abs(yb-yes2)));
set(abb,'linewidth',2); axis([0,1,-15,-9]); set(gca,'fontsize',18); print -depsc bsp22.eps;
hold on
abb=plot(x, log10(abs(yb-yespr)));
set(abb,'linewidth',2); axis([0,1,-15,-9]); set(gca,'fontsize',18); print -depsc bsp22.eps;
title('Error for: ESPIRA II - blue, ESPRIT - red')
hold off


figure
pos = [-1 -1 2 2]; 
rectangle('Position',pos,'Curvature',[1 1])
axis equal
hold on
plot(pol.^(1/(sf*b)),'+','MarkerSize',12)
hold on
pos = [-1 -1 2 2]; 
rectangle('Position',pos)
title('Normalized knots computed with ESPIRA I')
hold off

figure
pos = [-0.09 -0.09 0.18 0.18]; 
rectangle('Position',pos,'Curvature',[1 1])
axis equal
hold on
plot(g,'+','MarkerSize',12)
hold on
pos = [-0.09 -0.09 0.18 0.18]; 
rectangle('Position',pos)
title('Weights computed with ESPIRA I')
hold off




function f=statio(g1,a1,x)
[N,~]=size(g1);
f=0;
for k=1:N
f=f+g1(k)*exp(a1(k)*x);
end
end


function f=statio1(g1,a1,x)
[N,~]=size(g1);
f=0;
for k=1:N
f= f+ abs(g1(k))*exp(real(a1(k))*x);
end
end



